********************************************************************************
*This file generates Figure 1 and Table 3 in the paper.



global date 		= "29Oct21"
global workdir 		= "B:\CBDC-Li"
global data 		= "$workdir\data"
global results 		= "$workdir\results\\$date"


clear all
set maxvar 120000



cap mkdir "$workdir\\results\\$date\\CBDC prediction"


********************************************************************************
*FIGURE 1 - AGGREGATE CBDC SHARES 
********************************************************************************

//load data 
forval yr = 2017(1)2017 {

	clear
	use  "$workdir/data/counterfactual analysis/$date/meanshares_dshares_nest_d_cbdc_design_real_real_rho0_gamma0_eta0_year`yr'.dta", clear

	
	foreach model in nest_d_cbdc nest_c_cbdc {
		foreach design in design_real design_d design_c {
			local attr = subinstr("`design'", "design_", "", .)
			
			foreach rho in 0  {
				foreach gamma in 0 0.2 0.4 0.6 0.8 1 {
					//local eta  `gamma'
					foreach eta in 0 0.2 0.4 0.6 0.8 1 {
						append using "$workdir/data/counterfactual analysis/$date/meanshares_dshares_`model'_`design'_`attr'_rho`rho'_gamma`gamma'_eta`eta'_year`yr'.dta"

					}
				}
				
			}
		}
	}

	duplicates drop	         

	
	//plot graphs conditional on both eta and gamma, x-axis: Group gamma and eta together     
	gen index = . 
	gen index_d = . 
	gen index_c = .
	forval value = 0(0.2)1 {
		replace index = `value' if eta == float(`value') & gamma == float(`value') 
		//de-clutter the 3 lines
		replace index_d = `value' - 0.025 if eta == float(`value') & gamma == float(`value') 
		replace index_c = `value' + 0.025 if eta == float(`value') & gamma == float(`value') 
	}	
}

*-------------------------------------------------------------------------------
//plot aggregate CBDC shares against eta and gamma   

forval yr = 2017(1)2017 {
	
	foreach nest in d_cbdc  {
		local model "nest_`nest'"

		//v is the variable on y-axis
		foreach v in as_CBDC {

			foreach rho in 0 {
				
				//3 designs on one graph	
				graph twoway (scatter `v' index_d if rho == float(`rho') & attr_d != . & nest == "`model'", msize(medsmall) m(circle) mcolor(pink)) (rcap `v'_lo `v'_hi index_d if rho == float(`rho') & attr_d != . & nest == "`model'", lcolor(pink) lwidth(0.6)) (scatter `v' index_c if rho == float(`rho') & attr_c != . & nest == "`model'", msize(medsmall) m(s) mcolor(blue)) (rcap `v'_lo `v'_hi index_c if rho == float(`rho') & attr_c != . & nest == "`model'", lcolor(blue) lwidth(0.6)) (scatter `v' index if rho == float(`rho') & attr_real != . & nest == "`model'", msize(medsmall) m(triangle) mcolor(green)) (rcap `v'_lo `v'_hi index if rho == float(`rho') & attr_real != . & nest == "`model'", lcolor(green) lwidth(0.6)),   xsize(3) ysize(3.2) xlabel(,labsize(medsmall)) ylabel(,labsize(medsmall))   xtitle("CBDC-specific effects" , size(medium)) legend(label(1 "Deposit design ({bf:x}{subscript:cbdc} = {bf:x}{subscript:d})")   label(2 "95% CI") label(3 "Cash design ({bf:x}{subscript:cbdc} = {bf:x}{subscript:c})")  label(4 "95% CI") label(5 "Baseline design ({bf:x}{subscript:cbdc} = {bf:x}{subscript:base})") label(6 "95% CI") cols(2) pos(6) ring(3)  size(medsmall))
				graph export "$workdir/results/$date/CBDC prediction/`yr'/`v'_against_gamma_eta_rho`rho'_`model'_attr_d_c_real.pdf", as(pdf) replace
				
			}
			
		}
	}
}


********************************************************************************
*TABLE 3 - IMPACTS OF ATTRIBUTE CHANGES 
********************************************************************************
//LOGIT PREDICTION ONLY

//LOAD DATA 
local yr = 2017
cap mkdir  "$workdir/results/$date/CBDC prediction//`yr'"

clear
use  "$workdir/data/counterfactual analysis/$date/meanshares_dshares_nest_d_cbdc_design_real_real_rho0_gamma0_eta0_year`yr'.dta", clear

foreach model in nest_d_cbdc {
	foreach design in design_real {
		local attr = subinstr("`design'", "design_", "", .)
		foreach rho in 0    {
			foreach gamma in 0 0.2 0.4 0.6 0.8 1 {
				foreach eta in 0 0.2 0.4 0.6 0.8 1 {
					append using "$workdir/data/counterfactual analysis/$date/meanshares_dshares_`model'_`design'_`attr'_rho`rho'_gamma`gamma'_eta`eta'_year`yr'.dta"
				}
			}
		}
	}
}
duplicates drop	 

*-------------------------------------------------------------------------------
//PLOT

foreach  model in nest_d_cbdc  {
 
	capture file close myfile
	file open myfile using "$workdir/results/$date/CBDC prediction/`yr'/table_attr_changes_pct_changes_in_scbdc_`model'_rho0.tex", write replace
	file write myfile "  Design	attribute  &  Change in attribute  &   \% change in $ s_{cbdc} $    \\ "   _n 
	file write myfile "    &  relative to baseline design   &   logit model   \\ "   _n 
	file write myfile "\midrule"    _n
	
	//change in attributes relative to baseline design
	local design  design_real 
	
	
	//Table ROW 1
	local di_stat_list " Interest rate  &  0\% $ \rightarrow $ 0.1\%    " 
	forval val = 0(1)1 { 
		qui su dlp_CBDC  if attr_rate == float(0.1)  &  rho == float(0) & eta == `val' & gamma == `val'  &  design == "`design'"  &  nest == "`model'" 
		local mean = r(mean)
		local mean_trim`val' = round(`mean')
	}
	local di_stat_list "`di_stat_list' & `mean_trim1' to `mean_trim0' "

	file write myfile " `di_stat_list'  \\ [0.2em] "   _n


	//TABLE ROW 2
	local di_stat_list " Budgeting usefulness  &  0.7 $ \rightarrow $ 0  " 
	//baseline logit model prediction  
	forval val = 0(1)1 { 
		qui su dlp_CBDC  if attr_budget == float(0)  &  rho == float(0) & eta == `val' & gamma == `val'  &  design == "`design'"  &  nest == "`model'" 
		local mean = r(mean)
		local mean_trim`val' = round(`mean')
	}
	local di_stat_list "`di_stat_list' & `mean_trim1' to `mean_trim0' "

	file write myfile " `di_stat_list'  \\ [0.2em] "   _n
	
	
	//Table ROW 3
	local di_stat_list " Anonymity  &  0.7 $ \rightarrow $ 0  " 
	forval val = 0(1)1 { 
		qui su dlp_CBDC  if attr_anonym == float(0)  &  rho == float(0) & eta == `val' & gamma == `val'  &  design == "`design'"  &  nest == "`model'" 
		local mean = r(mean)
		local mean_trim`val' = round(`mean')
	}
	local di_stat_list "`di_stat_list' & `mean_trim1' to `mean_trim0'  "

	file write myfile " `di_stat_list'  \\ [0.2em] "   _n
	
	
	//Table ROW 4
	local di_stat_list " Bundling of bank service  &  0 $ \rightarrow $ 1 " 
	forval val = 0(1)1 { 
		qui su dlp_CBDC  if attr_bundling == float(1)  &  rho == float(0) & eta == `val' & gamma == `val'  &  design == "`design'"  &  nest == "`model'" 
		local mean = r(mean)
		local mean_trim`val' = round(`mean')
	}
	local di_stat_list "`di_stat_list' & `mean_trim1' to `mean_trim0'  "

	file write myfile " `di_stat_list'  \\ [0.2em] "   _n
	file close myfile

}












