********************************************************************************
*This file is used to generate Table 1 in the paper. 



global date 		= "29Oct21"
global workdir 		= "B:\CBDC-Li"
global data 		= "$workdir\data"
global results 		= "$workdir\results\\$date"


clear all
set maxvar 120000


cap mkdir "$workdir\\results\\$date\\demand estimation"



********************************************************************************
*Table 1 - Demand estimation - Baseline specification
********************************************************************************
clear
use "$data/Data_varlabel_edited_v4_MOP13.dta"



global attrlist  rate  bundling  reldif_costopp_dc  reldif_ease_dc reldif_secur_dc anonym06 budget06 sn_loc_online_mop frac_card_unaccep
global demolist  att_risk1  att_debt1  ind_behinddebt  i.ageheadcat5  i.ind_hldincome  i.hldsizecat  i.educhead  ind_rent  female  ind_inetwork  ind_rural  ind_TD  ind_RBC ind_LAU  ind_OT i.year i.regioncat5 
reg  log_dep3_cash  ${attrlist}  ${demolist}  if inrange(year, 2010, 2017),  vce(robust)
cap drop sample
gen sample = 1 if e(sample)

*-------------------------------------------------------------------------------
//Save the estimates on product attributes in a table 

capture file close myfile
file open myfile using "$results\\demand estimation/table_reg_logdep3cash_baseline_10t17.tex", write replace
file write myfile "    &  Interest rate & Bundling &  Cost & Ease & Security & Anonymity & Budgeting & Online &  Card unacceptance  \\ "   _n 
file write myfile "\midrule"    _n

global varlist ${attrlist}

//first row - coefficients 
local di_stat_list "   Estimates " 
foreach var of global varlist {
	local  param  = _b[`var']
	local  tstat  = `param'/_se[`var']
	local  param_trim = strltrim("`: display %9.3f `param''")
	if abs(`tstat') >= 2.575 {
		local  di_stat_list "`di_stat_list' &  $ `param_trim'^{***} $ "
	}
	
	else if  abs(`tstat') >= 1.96 & abs(`tstat') < 2.575 {
		local  di_stat_list "`di_stat_list' &  $ `param_trim'^{**} $ "
	}
	else if  abs(`tstat') >= 1.645 & abs(`tstat') < 1.96 {
		local  di_stat_list "`di_stat_list' &  $ `param_trim'^{*} $ "
	}
	else {
		local  di_stat_list "`di_stat_list' & `param_trim' "
	}
	
}
file write myfile " `di_stat_list'  \\ [0.2em] "   _n

//second row - standard errors
local di_stat_list "  " 
foreach var of global varlist {
	local  standard_error  = _se[`var']
	local  se_trim = strltrim("`: display %9.3f `standard_error''")
	local  di_stat_list "`di_stat_list' & (`se_trim') "
}
file write myfile " `di_stat_list'  \\ [0.2em] "   _n
file close myfile






 
